/* 
 * Copyright 2004-2009 Rob Beverly
 * Copyright 2015-2016 The Regents of the University of California
 * All rights reserved.
 * 
 * This file is part of Spoofer.
 * 
 * Spoofer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Spoofer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Spoofer.  If not, see <http://www.gnu.org/licenses/>.
 */

/****************************************************************************
   Program:     $Id: spoof.h,v 1.68 2016/06/07 18:36:41 kkeys Exp $
   Date:        $Date: 2016/06/07 18:36:41 $
   Description: IP spoofer tester headers
****************************************************************************/

#if HAVE_CONFIG_H
  #include "config.h"
#endif

#ifndef _COMMON_SPOOF_H_
#define _COMMON_SPOOF_H_ 1

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wshadow"
#include "spoofer.pb.h"
#pragma GCC diagnostic pop

#ifdef _WIN32
  #define NOGDI // prevents <wingdi.h> from polluting our namespace
  #include <winsock2.h>
  #include <ws2tcpip.h>
  #include <unistd.h>
  #include <stdint.h>
  #undef HAVE_NETINET_IP_H
  #undef HAVE_IN_ADDR_T
#else
  #include <sys/types.h>
  #ifndef __FAVOR_BSD
    #define  __FAVOR_BSD
  #endif
  #include <unistd.h>
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
  #include <netinet/in_systm.h>
  #include <netinet/ip.h>
  #ifdef HAVE_NETINET_IP6_H
    #include <netinet/ip6.h>
  #endif
  #include <netdb.h>
#endif

#ifdef __GNUC__
# define NORETURN                 __attribute__((__noreturn__)) void
# define UNUSED                   __attribute__((__unused__))
# define USED                     __attribute__((__used__))
# define FORMAT_PRINTF(fmt, var)  __attribute__((__format__(printf, fmt, var)))
#else
# define NORETURN                 void
# define UNUSED                   /* empty */
# define USED                     /* empty */
# define FORMAT_PRINTF(fmt, var)  /* empty */
#endif

/* The UDP header layout is an universal standard, not OS-specific, so it makes
 * more sense to define our own copy than to depend on differently-named
 * definitions provided by the host OS. */
struct udphdr
{
  uint16_t uh_sport;           /* source port */
  uint16_t uh_dport;           /* destination port */
  uint16_t uh_ulen;            /* udp length */
  uint16_t uh_sum;             /* udp checksum */
};

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <errno.h>
#include "messages.h"

#define VERBOSE 0
//#define REGRESS 1
#ifdef REGRESS
#undef VERSION
#define VERSION 99
#endif

#define BUFSIZE 1500
#define SMALLBUF 80
#define BIGBUF 9000
#define MAXMSGSIZE 4096
#define LISTENQ 1024

#define TTL_OUT 64
#define SRC_PORT 5353
#define CLIENT_TIMEOUT 60

#define TEST_SERVER "192.172.226.236"
#define TEST_SERVER6 "2001:48d0:101:501::236"
#define TEST_REPORT_HOST "spoofer-test.caida.org"

#define PROD_SERVER "192.172.226.242"
#define PROD_SERVER6 "2001:48d0:101:501::242"
#define PROD_REPORT_HOST "spoofer.caida.org"

#if 0
// testing
 #define SERVER TEST_SERVER
 #define SERVER6 TEST_SERVER6
 #define REPORT_HOST TEST_REPORT_HOST
#else
// production
 #define SERVER PROD_SERVER
 #define SERVER6 PROD_SERVER6
 #define REPORT_HOST PROD_REPORT_HOST
#endif

#define PBS_MAGIC "SPOOFER"

#if 0
  #define SERV_PORT 5353
  #define REPORT_PORT 8080
  #define ICMP_DIVERT_PORT 3002 
  #define UDP_DIVERT_PORT 3003 
  #define REPORT_HOST "spoofer-test.XXX.XXX"
#else
  #define SERV_PORT 53
  #define REPORT_PORT 80
  #define ICMP_DIVERT_PORT 2002 
  #define UDP_DIVERT_PORT 2003 
#endif

#ifndef FALSE
 #define FALSE 0
 #define TRUE !FALSE
#endif

#define SENDTO_ERROR -2
#define WINRAWSOCK_ERROR -3
#ifndef SOCKET_ERROR
 #define SOCKET_ERROR -4
#endif

#ifndef LITTLE_ENDIAN
  #define LITTLE_ENDIAN   1234
  #define BIG_ENDIAN      4321
#endif
#ifndef BYTE_ORDER
  #if HAVE_BIG_ENDIAN
    #define BYTE_ORDER    BIG_ENDIAN
  #else
    #define BYTE_ORDER    LITTLE_ENDIAN
  #endif
#endif

#ifndef HAVE_NETINET_IP_H
/*
 * Structure of an internet header, naked of options.
 */
struct ip {
#ifdef _IP_VHL
        u_char  ip_vhl;                 /* version << 4 | header length >> 2 */
#else
	/* Note: the base type of ip_hl and ip_v must be an 8-bit type, since
	 * some compilers (at least mxe/mingw gcc 5.1.0) allocate the full
	 * size of the base type. */
#if BYTE_ORDER == LITTLE_ENDIAN
        u_char  ip_hl:4,                /* header length */
                ip_v:4;                 /* version */
#endif
#if BYTE_ORDER == BIG_ENDIAN
        u_char  ip_v:4,                 /* version */
                ip_hl:4;                /* header length */
#endif
#endif /* not _IP_VHL */
        u_char  ip_tos;                 /* type of service */
        u_short ip_len;                 /* total length */
        u_short ip_id;                  /* identification */
        u_short ip_off;                 /* fragment offset field */
#define IP_RF 0x8000                    /* reserved fragment flag */
#define IP_DF 0x4000                    /* dont fragment flag */
#define IP_MF 0x2000                    /* more fragments flag */
#define IP_OFFMASK 0x1fff               /* mask for fragmenting bits */
        u_char  ip_ttl;                 /* time to live */
        u_char  ip_p;                   /* protocol */
        u_short ip_sum;                 /* checksum */
        struct  in_addr ip_src,ip_dst;  /* source and dest address */
};
#endif /* HAVE_NETINET_IP_H */

enum { assert__struct_ip_is_20_bytes = 1/(sizeof(struct ip) == 20)};

enum IPv { IPv4 = 4, IPv6 = 6 };

#ifndef HAVE_NETINET_IP6_H
/* RFC 3542 */
struct ip6_hdr {
    union {
        struct ip6_hdrctl {
            uint32_t ip6_un1_flow; /* 4 bits version, 8 bits TC, 20 bits
                                      flow-ID */
            uint16_t ip6_un1_plen; /* payload length */
            uint8_t  ip6_un1_nxt;  /* next header */
            uint8_t  ip6_un1_hlim; /* hop limit */
        } ip6_un1;
        uint8_t ip6_un2_vfc;     /* 4 bits version, top 4 bits
                                    tclass */
    } ip6_ctlun;
    struct in6_addr ip6_src;   /* source address */
    struct in6_addr ip6_dst;   /* destination address */
};

#define ip6_vfc   ip6_ctlun.ip6_un2_vfc
#define ip6_flow  ip6_ctlun.ip6_un1.ip6_un1_flow
#define ip6_plen  ip6_ctlun.ip6_un1.ip6_un1_plen
#define ip6_nxt   ip6_ctlun.ip6_un1.ip6_un1_nxt
#define ip6_hlim  ip6_ctlun.ip6_un1.ip6_un1_hlim
#define ip6_hops  ip6_ctlun.ip6_un1.ip6_un1_hlim

#endif /* HAVE_NETINET_IP6_H */

#ifndef HAVE_IN_ADDR_T
 typedef unsigned int in_addr_t;
#endif

#ifndef HAVE_INET_PTON
int inet_pton(int af, const char *src, void *dst);
#endif
#ifndef HAVE_INET_NTOP
const char *inet_ntop(int af, const void *src, char *dst, size_t cnt);
#endif

#ifndef HAVE_STRUCT_SOCKADDR_IN6
struct in6_addr
{
  uint8_t 	  s6_addr[16];
};

struct sockaddr_in6
{
  sa_family_t	  sin6_family;		/* AF_INET6 */
  in_port_t	  sin6_port;		/* Port number. */
  uint32_t	  sin6_flowinfo;	/* Traffic class and flow inf. */
  struct in6_addr sin6_addr;		/* IPv6 address. */
  uint32_t	  sin6_scope_id;	/* Set of interfaces for a scope. */
};
#endif

#ifndef IPVERSION
 #define IPVERSION 4
#endif

#ifndef IP_DF
 #define IP_DF 0x4000
#endif

#ifndef AF_INET6
 #define AF_INET6 10
#endif


// get a ref to the port part of a sockaddr_{in,in6} (in network order)
static inline uint16_t &sa_port(void *sa)
{
    if (((struct sockaddr *)(sa))->sa_family == AF_INET)
        return ((struct sockaddr_in *)(sa))->sin_port;
    if (((struct sockaddr *)(sa))->sa_family == AF_INET6)
        return ((struct sockaddr_in6 *)(sa))->sin6_port;
    return ((struct sockaddr_in *)(sa))->sin_port; // shouldn't happen
}

// get the port part of a const sockaddr_{in,in6} (in network order)
static inline uint16_t const_sa_port(const void *sa)
{
    if (((const struct sockaddr *)(sa))->sa_family == AF_INET)
        return ((const struct sockaddr_in *)(sa))->sin_port;
    if (((const struct sockaddr *)(sa))->sa_family == AF_INET6)
        return ((const struct sockaddr_in6 *)(sa))->sin6_port;
    return 0;
}

/* get a pointer to the IP address part of a sockaddr_{in,in6} */
static inline char *sa_ipaddr(void *sa)
{
    if (((struct sockaddr *)sa)->sa_family == AF_INET)
        return (char*)&((struct sockaddr_in *)sa)->sin_addr;
    if (((struct sockaddr *)sa)->sa_family == AF_INET6)
        return (char*)&((struct sockaddr_in6 *)sa)->sin6_addr;
    return NULL;
}

/* get a pointer to the IP address part of a const sockaddr_{in,in6} */
static inline const char *const_sa_ipaddr(const void *sa)
{
    if (((const struct sockaddr *)sa)->sa_family == AF_INET)
        return (const char*)&((const struct sockaddr_in *)sa)->sin_addr;
    if (((const struct sockaddr *)sa)->sa_family == AF_INET6)
        return (const char*)&((const struct sockaddr_in6 *)sa)->sin6_addr;
    return NULL;
}

#define sa_len(sa)   (\
    (((const struct sockaddr *)(sa))->sa_family == AF_INET) ? \
        sizeof(struct sockaddr_in) : \
    (((const struct sockaddr *)(sa))->sa_family == AF_INET6) ? \
        sizeof(struct sockaddr_in6) : \
    0 \
)

static inline unsigned int addrlen(const int &family)
{
    return
	(family == AF_INET6) ? IPV6ADDRLEN :
	(family == AF_INET) ? IPV4ADDRLEN :
	0;
}

static inline int ipvtofamily(const IPv &ipv) {
    return
	(ipv == IPv6) ? AF_INET6 :
	(ipv == IPv4) ? AF_INET :
	0;
}


/* get pointer to address within an old-style right-aligned address buffer */
#define right_addr(family, addr) \
    (((unsigned char*)addr) + IPV6ADDRLEN - addrlen(family))


/* prettyprint */
#define NIPQUAD(addr) \
  ((unsigned char *)&addr)[0], \
  ((unsigned char *)&addr)[1], \
  ((unsigned char *)&addr)[2], \
  ((unsigned char *)&addr)[3]

/* error messages */
#define tracefunc() \
    do { \
	if (verbosity >= DEVELOP) \
	    fprintf(stdout, "\t>> %s:%s():%d\n", __FILE__, __FUNCTION__, __LINE__); \
    } while (0)

#define pprintf(prefix, ...) \
    do { \
	fprintf(stdout, "*** %s: ", prefix); \
	fprintf(stdout, __VA_ARGS__); \
	fprintf(stdout, "\n"); \
    } while (0)
#define info(...)   pprintf("Info", __VA_ARGS__)    // not displayed in gui
#define notice(...) pprintf("Notice", __VA_ARGS__)  // displayed in gui
#define warn(...)   pprintf("Warning", __VA_ARGS__) // displayed in gui
#define severe(...) pprintf("Error", __VA_ARGS__)   // displayed in gui

#define debug(level, ...) \
    do { if (verbosity >= level) fprintf(stdout, __VA_ARGS__); } while (0)

enum debugLevel {OFF, LOW, HIGH, DEVELOP};
extern int verbosity;

typedef void Sigfunc(int);

/* net.cc */
extern int spoof_layer[];
typedef struct probe_info probe_info_t;
const char *sock_errmsg(int err);
const char *sock_last_errmsg(void);
int craftPacket(unsigned char *, unsigned char *, int, 
    struct sockaddr *, struct sockaddr *, u_char, unsigned short);
void craftProbe(unsigned char *buf, bool spoof, const SpooferSpoofSchedule::Item *item);
unsigned short in_cksum(unsigned short *, int);
char * genSequence(char *, int);
Sigfunc *Signal(int signo, Sigfunc *func);
void sig_chld(int signo);
size_t Readn(int fd, void *ptr, size_t nbytes, time_t timeout);
void Writen(int fd, const void *ptr, size_t nbytes);
ssize_t Readline(int fd, void *ptr, size_t maxlen);
ssize_t readn(int fd, void *vptr, size_t n, time_t wait);
bool spoof_layer2possible(const IPv &ipv);
int newSocket(int, int, int);
int getSockTTL(int family, int sock, int *val);
int setSockTTL(int family, int sock, int val);
int optSocketHDRINCL(int);
struct probe_info *new_probe_info(int proto, bool spoof, int sock,
    struct sockaddr *src_sa, struct sockaddr *dst_sa);
int socketSend(int, const void *, int, struct probe_info *);
int socketSendSpoof(int, int, const void *, int, struct probe_info *);
void free_probe_info(struct probe_info *pinfo, bool skipSniffing = false);
const char *sa_ntop(const struct sockaddr *sa, char *dst, size_t len);

/* util.cc */
unsigned long byte32Swap(unsigned long);
unsigned short byte16Swap(unsigned short);
void binDump(const unsigned char *msg, int len, int pretty);
inline void binDump(const char *msg, int len, int pretty) { binDump((const unsigned char *)msg, len, pretty); }
void win_init(void);
void msleep(int msec);
void spoofSleep(void);

int spoofer_snprintf(char *buf, size_t size, const char *fmt, ...) FORMAT_PRINTF(3, 4);
#ifdef _WIN32
#define snprintf spoofer_snprintf // replace Windows' broken snprintf()
#endif
int is_interactive(void);
NORETURN exitpause(int status);
NORETURN fatal(const char *, ...) FORMAT_PRINTF(1, 2);
NORETURN pfatal(const char *, ...) FORMAT_PRINTF(1, 2);
void streamSetup(void);
char *timestring(void); // formerly timestamp()
uint32_t n6ton4(const unsigned char *addr);
uint32_t n6toh4(const unsigned char *addr);
int n4ton6(const uint32_t ip, unsigned char *addr);
int h4ton6(const uint32_t ip, unsigned char *addr);
int n6tohexstring(const unsigned char *addr, unsigned char *res);
int isIPv4(const unsigned char *addr);
int isIPv6(const unsigned char *addr);
char *printIPv6(const unsigned char *a);
int isIPv6Equal(const unsigned char *a1, const unsigned char *a2);
const char *strstrPortable(const char *a, const char *b, size_t n);

/* messages.cc */
const char *pb_ntop(int family, const std::string &addr, char *buf, size_t buflen);
const char *pb_ntop(int family, const std::string &addr);
void printServerMsg(const SpooferServerMsg &msg, IPv ipv);
void printClientMsg(const SpooferClientMsg &msg, IPv ipv);
const char *reportStatusToStr(SpooferReportStatus status);
const char *spoofStatusToStr(SpooferSpoofReport::Item::Status status);
const char *summaryResultToStr(SpooferResultSummary::Result result);
#if 0
char *msg_type_name(uint8_t typ);
void printAddressList(struct in_addr *iplist, int num);
void printMsg(unsigned char *, int);
spoof_msg_t *readMsg(int sock, unsigned char **message, enum msg_types);
void defaultMsg(spoof_msg_t *msg);
int errorRequest(unsigned char *);
int helloRequest(unsigned char *);
int reportRequest(unsigned char *msg, unsigned char *seqno, 
    unsigned char *src_addr, unsigned char *dst_addr, int nat, int ipv6);
int doneRequest(unsigned char *msg);
int probeRequest(unsigned char *msg, int v6);
int readProbeList(unsigned char *buf, test_t **testlistptr);
int traceFilterRequest(unsigned char *msg);
#endif

#endif /* _COMMON_SPOOF_H_ */
