/* 
 * Copyright 2015-2016 The Regents of the University of California
 * All rights reserved.
 * 
 * This file is part of Spoofer.
 * 
 * Spoofer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Spoofer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Spoofer.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include "spoof_qt.h"
#include <QApplication>
#include <QCommandLineParser>
#include <QMessageBox>
#include <QtDebug>
#include <mainwindow.h>
#include "../../config.h"
static const char cvsid[] USED = "$Id: main.cpp,v 1.15 2016/06/16 18:48:18 kkeys Exp $";

static void msgboxLogHandler(QtMsgType type, const QMessageLogContext &ctx,
    const QString &msg)
{
    SpooferBase::logHandler(type, ctx, msg);
    if (type == QtCriticalMsg || type == QtFatalMsg || type == QtWarningMsg) {
	QMessageBox mb;
	mb.setMinimumWidth(400); // ignored?!
	mb.setIcon(type == QtWarningMsg ? QMessageBox::Warning : QMessageBox::Critical);
	// note: setWindowTitle() would be ignored on OSX
	QString text = MainWindow::mainTitle %
	    (type == QtWarningMsg ? QSL(": Warning") : QSL(": Error"));
	text += QSL(" ").repeated(60 - text.length()); // force min width
	mb.setText(text);
	mb.setInformativeText(msg);
	mb.setStandardButtons(QMessageBox::Ok);
	mb.exec();
    }
}

int main(int argc, char *argv[])
{
    int retval = -1;
    QApplication app(argc, argv);
    MainWindow window;

    QCommandLineParser clp;
    if (!SpooferBase::parseCommandLine(clp,
	QSL("Spoofer scheduler graphical user interface")))
    {
	retval = 1;
	goto done;
    }

    qInstallMessageHandler(::msgboxLogHandler);

    if (!window.initConfig()) {
	retval = 1;
	goto done;
    }

    window.init();
    window.show();

    retval = app.exec();
done:
    qDebug() << "exit " << retval;
    return retval;
}
